"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
;
const AUTOMATA_REDUCER_DEFAULTS = {
    key: 'state',
    toStandardAction: identity,
    operator: identity
};
function createAutomataReducer(automata, init, opts) {
    if (typeof opts === 'string') {
        return createAutomataReducer(automata, init, { key: opts });
    }
    const { key, toStandardAction, operator } = Object.assign({}, AUTOMATA_REDUCER_DEFAULTS, opts);
    const _automata = preprocess(automata, init, key, operator);
    return function (previous, event) {
        const { type } = toStandardAction(event);
        const reducer = _automata[type];
        return !reducer ? previous : reducer(previous, event);
    };
}
exports.default = createAutomataReducer;
function preprocess(spec, init, key, operator) {
    const _automata = Object.keys(spec).reduce(function (automata, state) {
        const events = spec[state];
        return Object.keys(events).reduce(function (automata, type) {
            const reducers = []
                .concat(events[type])
                .map(withAutomataReducer);
            automata[type] = Object.assign({}, automata[type], { [state]: reducers });
            return automata;
        }, automata);
    }, {});
    return Object.keys(_automata).reduce(function (eventTypeToReducerMap, type) {
        const stateToReducersMap = _automata[type];
        eventTypeToReducerMap[type] = operator(reducer);
        return eventTypeToReducerMap;
        function reducer(previous = {}, event = {}) {
            let state = previous[key]
                ? previous
                : Object.assign({}, previous, { [key]: init });
            const reducers = stateToReducersMap[state[key]] || [];
            let i = reducers.length;
            while (i--) {
                state = reducers[i](state, event);
            }
            return state;
        }
    }, {});
    function withAutomataReducer(reducer) {
        return !isString(reducer)
            ? reducer
            : function (state) {
                return state && state[key] === reducer
                    ? state
                    : Object.assign({}, state, { [key]: reducer });
            };
    }
}
function isString(v) {
    return typeof (v && v.valueOf()) === 'string';
}
function identity(v) {
    return v;
}
